use actix_web::{post, web, HttpResponse, Responder};
use crate::models::{RegistrationData, LoginData, LoginResponse};
use crate::error::AppError;
use bcrypt::{hash, verify, DEFAULT_COST};
use sqlx::PgPool;

#[post("/rejestracja")]
pub async fn register(
    form: web::Json<RegistrationData>,
    pool: web::Data<PgPool>,
) -> Result<HttpResponse, AppError> {
    if form.haslo.len() < 8 {
        return Err(AppError::BadRequest("Hasło musi mieć minimum 8 znaków".to_string()));
    }

    if form.haslo != form.confirm_password {
        return Err(AppError::BadRequest("Hasła nie są identyczne".to_string()));
    }

    let hashed_password = hash(&form.haslo, DEFAULT_COST).map_err(|_| AppError::InternalServerError("Błąd serwera".to_string()))?;

    sqlx::query!(
        r#"
        INSERT INTO uzytkownicy (email, haslo, imie)
        VALUES ($1, $2, $3)
        "#,
        form.email,
        hashed_password,
        form.imie
    )
    .execute(pool.get_ref())
    .await
    .map_err(|e| {
        if e.to_string().contains("duplicate key value") {
            AppError::BadRequest("Email jest już zarejestrowany".to_string())
        } else {
            AppError::InternalServerError("Błąd serwera".to_string())
        }
    })?;

    Ok(HttpResponse::Created().body("Konto utworzone pomyślnie"))
}

#[post("/login")]
pub async fn login(
    form: web::Json<LoginData>,
    pool: web::Data<PgPool>,
) -> Result<HttpResponse, AppError> {
    let user = sqlx::query!(
        "SELECT id, haslo, imie FROM uzytkownicy WHERE email = $1",
        form.email
    )
    .fetch_optional(pool.get_ref())
    .await
    .map_err(|_| AppError::InternalServerError("Błąd serwera".to_string()))?
    .ok_or_else(|| AppError::Unauthorized("Nieprawidłowe dane".to_string()))?;

    if verify(&form.haslo, &user.haslo).map_err(|_| AppError::InternalServerError("Błąd serwera".to_string()))? {
        let dummy_token = format!("user-{}-token", user.id);
        Ok(HttpResponse::Ok().json(LoginResponse {
            token: dummy_token,
            imie: user.imie,
        }))
    } else {
        Err(AppError::Unauthorized("Nieprawidłowe hasło".to_string()))
    }
}

