use actix_web::{get, post, delete, web, HttpResponse, HttpRequest, Responder};
use crate::models::{CartItem, CartItemResponse};
use crate::error::AppError;
use sqlx::PgPool;

async fn validate_token(token: Option<&str>) -> Result<i32, AppError> {
    let raw_token = token.ok_or(AppError::Unauthorized("Unauthorized".to_string()))?;
    let token = raw_token.trim_start_matches("Bearer ").trim();

    if token.starts_with("user-") {
        let user_id = token.replace("user-", "").replace("-token", "").parse()
            .map_err(|_| AppError::Unauthorized("Invalid token".to_string()))?;
        Ok(user_id)
    } else {
        Err(AppError::Unauthorized("Unauthorized".to_string()))
    }
}

fn get_token(req: &HttpRequest) -> Option<&str> {
    req.headers().get("Authorization").and_then(|h| h.to_str().ok())
}

#[get("/api/cart")]
pub async fn get_cart(
    req: HttpRequest,
    pool: web::Data<PgPool>,
) -> Result<HttpResponse, AppError> {
    let user_id = validate_token(get_token(&req)).await?;

    let cart_items = sqlx::query_as!(
        CartItemResponse,
        r#"
        SELECT
            k.book_id as "book_id!",
            k.quantity as "quantity!",
            b.tytul as "tytul!",
            b.cena as "cena!",
            COALESCE('/images/' || NULLIF(b.obraz_url, ''), '/images/placeholder.jpg') as "obraz_url!"
        FROM koszyk k
        JOIN ksiazki b ON k.book_id = b.id
        WHERE k.user_id = $1
        ORDER BY b.tytul"#,
        user_id
    )
    .fetch_all(pool.get_ref())
    .await
    .map_err(|_| AppError::InternalServerError("Błąd serwera".to_string()))?;

    Ok(HttpResponse::Ok().json(cart_items))
}

#[post("/api/add-to-cart")]
pub async fn add_to_cart(
    cart_item: web::Json<CartItem>,
    req: HttpRequest,
    pool: web::Data<PgPool>,
) -> Result<HttpResponse, AppError> {
    let token = get_token(&req);
    let user_id = validate_token(token).await?;

    sqlx::query!(
        "INSERT INTO koszyk (user_id, book_id, quantity)
        VALUES ($1, $2, $3)
        ON CONFLICT (user_id, book_id)
        DO UPDATE SET quantity = koszyk.quantity + $3",
        user_id,
        cart_item.book_id,
        cart_item.quantity
    )
    .execute(pool.get_ref())
    .await
    .map_err(|_| AppError::InternalServerError("Błąd serwera".to_string()))?;

    Ok(HttpResponse::Ok().json(serde_json::json!({"status": "success"})))
}

#[delete("/api/remove-from-cart/{book_id}")]
pub async fn remove_from_cart(
    req: HttpRequest,
    pool: web::Data<PgPool>,
    book_id: web::Path<i32>,
) -> Result<HttpResponse, AppError> {
    let user_id = validate_token(get_token(&req)).await?;
    let book_id = book_id.into_inner();

    sqlx::query!(
        "DELETE FROM koszyk WHERE user_id = $1 AND book_id = $2",
        user_id,
        book_id
    )
    .execute(pool.get_ref())
    .await
    .map_err(|_| AppError::InternalServerError("Błąd serwera".to_string()))?;

    Ok(HttpResponse::Ok().json(serde_json::json!({"status": "success"})))
}

