use actix_web::{get, web, HttpResponse, HttpRequest};
use serde::Serialize;
use sqlx::PgPool;
use crate::error::AppError;
use bigdecimal::BigDecimal;
use chrono::NaiveDateTime;

// Struktura reprezentująca pojedynczą pozycję zamówienia
#[derive(Serialize)]
pub struct OrderItem {
    pub tytul: String,
    pub autor: String,
    pub ilosc: i32,
    pub cena: BigDecimal,
}

// Struktura reprezentująca zamówienie z pozycjami
#[derive(Serialize)]
pub struct OrderWithItems {
    pub id: i32,
    pub data_zamowienia: NaiveDateTime,
    pub suma_totalna: BigDecimal,
    pub status: Option<String>,
    pub items: Vec<OrderItem>,
}

// Funkcja pomocnicza do walidacji tokenu
async fn validate_token(token: Option<&str>) -> Result<i32, AppError> {
    let raw_token = token.ok_or(AppError::Unauthorized("Unauthorized".to_string()))?;
    let token = raw_token.trim_start_matches("Bearer ").trim();

    if token.starts_with("user-") {
        let user_id = token.replace("user-", "").replace("-token", "").parse()
            .map_err(|_| AppError::Unauthorized("Invalid token".to_string()))?;
        Ok(user_id)
    } else {
        Err(AppError::Unauthorized("Unauthorized".to_string()))
    }
}

// Funkcja pomocnicza do pobierania tokenu z nagłówka
fn get_token(req: &HttpRequest) -> Option<&str> {
    req.headers().get("Authorization").and_then(|h| h.to_str().ok())
}

#[get("/api/order-history")]
pub async fn get_order_history(
    req: HttpRequest,
    pool: web::Data<PgPool>,
) -> Result<HttpResponse, AppError> {
    let user_id = validate_token(get_token(&req)).await?;

    let orders = sqlx::query!(
        r#"
        SELECT
            z.id as "id!",
            z.data_zamowienia as "data_zamowienia!",
            z.suma_totalna as "suma_totalna!",
            z.status,
            pz.ilosc as "ilosc!",
            pz.cena as "item_price!",
            k.tytul as "tytul!",
            k.autor as "autor!"
        FROM zamowienia z
        JOIN pozycje_zamowienia pz ON z.id = pz.zamowienie_id
        JOIN ksiazki k ON pz.book_id = k.id
        WHERE z.user_id = $1
        ORDER BY z.id DESC, k.tytul ASC
        "#,
        user_id
    )
    .fetch_all(pool.get_ref())
    .await
    .map_err(|_| AppError::InternalServerError("Błąd serwera".to_string()))?;

    let mut grouped_orders: std::collections::HashMap<i32, OrderWithItems> = std::collections::HashMap::new();
    for record in orders {
        let entry = grouped_orders.entry(record.id).or_insert(OrderWithItems {
            id: record.id,
            data_zamowienia: record.data_zamowienia,
            suma_totalna: record.suma_totalna,
            status: record.status,
            items: Vec::new(),
        });

        entry.items.push(OrderItem {
            tytul: record.tytul,
            autor: record.autor,
            ilosc: record.ilosc,
            cena: record.item_price,
        });
    }

    let result: Vec<OrderWithItems> = grouped_orders.into_values().collect();
    Ok(HttpResponse::Ok().json(result))
}

